Attribute VB_Name = "MSubclass"
  Option Explicit
  ' demo project showing how to impliment a basic subclass in VB
  ' by Bryan Stafford of New Vision Software - newvision@mvps.org
  ' this demo is released into the public domain "as is" without
  ' warranty or guaranty of any kind.  In other words, use at
  ' your own risk.
  
  ' See the comments at the end of this module for a brief explaination of
  ' what subclassing is.
  
  
  ' Please note that this is a very simplistic example of subclassing.
  ' For a more robust example, see my 'Advanced Subclassing' demo project.
  

  ' the messages that we want to subclass
  Public Const WM_WINDOWPOSCHANGED As Long = &H47
  Public Const WM_MOVE As Long = &H3&
  
  ' used to demonstate sending application defined messages
  Public Const NV_CUSTOM_MESSAGE As Long = &H8FF1&

  ' this var will hold a pointer to the original message handler so we MUST
  ' save it so that it can be restored before we exit the app.  if we don't
  ' restore it.... CRASH!!!!
  Private m_procOld As Long
  
  ' constant used to get the address of the window proc for the subclassed window
  Private Const GWL_WNDPROC As Long = (-4&)
    
  ' API call to alter the class data for this window
  Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hWnd&, _
                                                              ByVal nIndex&, ByVal dwNewLong&) As Long
                                                             
  ' the function we use to send our custom message to the window
  Public Declare Function PostMessage Lib "user32" Alias "PostMessageA" (ByVal hWnd&, _
                                                    ByVal wMsg&, ByVal wParam&, lParam As Any) As Long
  ' function used to call the next window proc in the "chain" for the subclassed window
  Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc&, _
                                                    ByVal hWnd&, ByVal Msg&, ByVal wParam&, ByVal lParam&) As Long
  
'WARNING!!!! WARNING!!!! WARNING!!!! WARNING!!!! WARNING!!!! WARNING!!!!
'
' Do NOT try to step through this function in debug mode!!!!
' You WILL crash!!!  Also, do NOT set any break points in this function!!!
' You WILL crash!!!  Subclassing is non-trivial and should be handled with
' EXTREAME care!!!
'
' There are ways to use a "Debug" dll to allow you to set breakpoints in
' subclassed code in the IDE but this was not implimented for this demo.
'
'WARNING!!!! WARNING!!!! WARNING!!!! WARNING!!!! WARNING!!!! WARNING!!!!
  
Public Function WindowProc(ByVal hWnd As Long, ByVal iMsg As Long, _
                                              ByVal wParam As Long, ByVal lParam As Long) As Long

  ' this is *our* implimentation of the message handling routine
  
  ' determine which message was recieved
  Select Case iMsg
    
    ' good old standard move message.  sent when the window is moved
    Case WM_MOVE
      frmSubclass.Form_WindowMoved
        
    ' grab the message that let's us know that the position has changed.
    ' notice that this is also fired when the windows ZOrder changes, that
    ' is when the window get's the focus and is brought to the front.
  '  Case WM_WINDOWPOSCHANGED
  '    frmSubclass.Form_WindowMoved
      
      
    ' our custom message
    Case NV_CUSTOM_MESSAGE
      Beep
      frmSubclass.Label1 = "Custom message recieved at " & Format$(Now, "hh:mm:ss") & " !"
      
  End Select
  
  ' pass all messages on to VB and then return the value to windows
  WindowProc = CallWindowProc(m_procOld, hWnd, iMsg, wParam, lParam)

End Function

Public Sub InitiateSubclass(ByVal hWnd&)

  ' take control of message processing by installing our message handling
  ' routine into the chain of message routines for the passed hWnd
  m_procOld = SetWindowLong(hWnd, GWL_WNDPROC, AddressOf MSubclass.WindowProc)

End Sub

Public Sub RemoveSubclass(ByVal hWnd&)

  ' give message processing control back to VB
  ' if you don't do this you WILL crash!!!
    Call SetWindowLong(hWnd, GWL_WNDPROC, m_procOld)

End Sub

' What is subclassing anyway?
'
' Windows runs on "messages".  A message is a unique value that, when
' recieved by a window or the operating system, tells either that
' something has happened and that an action of some sort needs to be
' taken.  Sort of like your nervous system passing feeling messages
' to your brain and the brain passing movement messages to your body.
'
' So, each window has what is called a message handler.  This is a
' function where all of the messages FROM Windows are recieved.  Every
' window has one.  This means every button, textbox, picturebox, form,
' etc...  Windows keeps track of where the message handler (called a
' WindowProc [short for PROCedure]) in a "Class" structure associated
' with each window handle (otherwise known as hWnd).
'
' What happens when a window is subclassed is that you insert a new
' window procedure in line with the original window procedure.  In other
' words, Windows sends the messages for the given window to YOUR WindowProc
' FIRST where you are responsible for handling any messages you want to
' handle.  Then you pass the remaining messages on to the default
' WindoProc.  So it looks like this:
'
'  Windows Message Sender --> Your WindowProc --> Default WindowProc
'
' A window can be subclassed MANY times so it could look like this:
'
'  Windows Message Sender --> Your WindowProc --> Another WindowProc _
'  --> Yet Another WindowProc --> Default WindowProc
'
' You can also change the order of when you respond to a message by
' where in your routine you pass the message on to the next WindowProc.
' Let's say that you want to draw something on the window AFTER the
' default WindowProc handles the WM_PAINT message.  This is easily done
' by calling the default proc before you do your drawing.   Like so:
'
' Public Function WindowProc(Byval hWnd, Byval etc....)
'
'   Select Case iMsg
'     Case SOME_MESSAGE
'       DoSomeStuff
'
'     Case WM_PAINT
'       ' pass the message to the defproc FIRST
'       Call CallWindowProc(m_procOld, hWnd, iMsg, wParam, lParam)
'
'       DoDrawingStuff ' <- do your drawing
'
'       WindowProc = nYourReturnVal ' <- retrun the desired value
'                                   '    to the system
'
'       Exit Function ' <- exit since we already passed the
'                     '    measage to the defproc
'
'   End Select
'
'   ' pass all messages on to VB and then return the value to windows
'   WindowProc = CallWindowProc(m_procOld, hWnd, iMsg, wParam, lParam)
'
' End Function
'
'
' This is just a basic overview of subclassing but I hope it helps if
' you were fuzzy about the subject before reading this.
'















